@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 Jeremy Lainé
+ * Copyright (C) 2010-2013 Jeremy Lainé
  * Contact: http://code.google.com/p/qdjango/
  *
  * This file is part of the QDjango Library.
@@ -24,6 +24,51 @@
 #include "QDjangoMetaModel.h"
 #include "QDjangoQuerySet_p.h"
 
+// python-compatible hash
+
+static long string_hash(const QString &s)
+{
+    if (s.isEmpty())
+        return 0;
+
+    const QByteArray a = s.toLatin1();
+    unsigned char *p = (unsigned char *) a.constData();
+    long x = *p << 7;
+    for (int i = 0; i < a.size(); ++i)
+        x = (1000003*x) ^ *p++;
+    x ^= a.size();
+    return (x == -1) ? -2 : x;
+}
+
+static long stringlist_hash(const QStringList &l)
+{
+    long x = 0x345678L;
+    long mult = 1000003L;
+    int len = l.size();
+    foreach (const QString &s, l) {
+        --len;
+        x = (x ^ string_hash(s)) * mult;
+        mult += (long)(82520L + len + len);
+    }
+    x += 97531L;
+    return (x == -1) ? -2 : x;
+}
+
+// django-compatible digest
+
+static QString stringlist_digest(const QStringList &l)
+{
+    return QString::number(labs(stringlist_hash(l)) % 4294967296L, 16);
+}
+
+enum ForeignKeyConstraint
+{
+    NoAction,
+    Restrict,
+    Cascade,
+    SetNull
+};
+
 class QDjangoMetaFieldPrivate : public QSharedData
 {
 public:
@@ -31,21 +76,25 @@ public:
 
     bool autoIncrement;
     QString db_column;
-    QString foreignModel;
+    QByteArray foreignModel;
     bool index;
     int maxLength;
     QByteArray name;
     bool null;
     QVariant::Type type;
     bool unique;
+    bool blank;
+    ForeignKeyConstraint deleteConstraint;
 };
 
 QDjangoMetaFieldPrivate::QDjangoMetaFieldPrivate()
-    : autoIncrement(false),
-    index(false),
-    maxLength(0),
-    null(false),
-    unique(false)
+    : autoIncrement(false)
+    , index(false)
+    , maxLength(0)
+    , null(false)
+    , unique(false)
+    , blank(false)
+    , deleteConstraint(NoAction)
 {
 }
 
@@ -89,6 +138,14 @@ QString QDjangoMetaField::column() const
     return d->db_column;
 }
 
+/*!
+    Returns true if this field is nullable.
+*/
+bool QDjangoMetaField::isNullable() const
+{
+    return d->null;
+}
+
 /*!
     Returns true if this is a valid field.
 */
@@ -97,6 +154,30 @@ bool QDjangoMetaField::isValid() const
     return !d->name.isEmpty();
 }
 
+/*!
+    Returns true if this field is auto incremented.
+*/
+bool QDjangoMetaField::isAutoIncrement() const
+{
+    return d->autoIncrement;
+}
+
+/*!
+    Returns true if this field is unique.
+*/
+bool QDjangoMetaField::isUnique() const
+{
+    return d->unique;
+}
+
+/*!
+    Returns true if this field can be empty.
+*/
+bool QDjangoMetaField::isBlank() const
+{
+    return d->blank;
+}
+
 /*!
     Returns name of this meta field.
 */
@@ -105,13 +186,21 @@ QString QDjangoMetaField::name() const
     return QString::fromLatin1(d->name);
 }
 
+/*!
+    Returns the max length of this field
+*/
+int QDjangoMetaField::maxLength() const
+{
+    return d->maxLength;
+}
+
 /*!
     Transforms the given field value for database storage.
 */
 QVariant QDjangoMetaField::toDatabase(const QVariant &value) const
 {
     if (d->type == QVariant::String && !d->null && value.isNull())
-        return QString("");
+        return QLatin1String("");
     else if (!d->foreignModel.isEmpty() && d->type == QVariant::Int && d->null && !value.toInt()) {
         // store 0 foreign key as NULL if the field is NULL
         return QVariant();
@@ -122,9 +211,9 @@ QVariant QDjangoMetaField::toDatabase(const QVariant &value) const
 static QMap<QString, QString> parseOptions(const char *value)
 {
     QMap<QString, QString> options;
-    QStringList items = QString::fromUtf8(value).split(' ');
+    QStringList items = QString::fromLatin1(value).split(QLatin1Char(' '));
     foreach (const QString &item, items) {
-        QStringList assign = item.split('=');
+        QStringList assign = item.split(QLatin1Char('='));
         if (assign.size() == 2) {
             options[assign[0].toLower()] = assign[1];
         } else {
@@ -134,26 +223,31 @@ static QMap<QString, QString> parseOptions(const char *value)
     return options;
 }
 
+static bool stringToBool(const QString &value)
+{
+    return value.toLower() == QLatin1String("true") || value == QLatin1String("1");
+}
+
 class QDjangoMetaModelPrivate : public QSharedData
 {
 public:
     QList<QDjangoMetaField> localFields;
-    QMap<QByteArray, QString> foreignFields;
+    QMap<QByteArray, QByteArray> foreignFields;
     QByteArray primaryKey;
     QString table;
+    QList<QByteArray> uniqueTogether;
 };
 
 /*!
-    Constructs a new QDjangoMetaModel by inspecting the given \a model instance.
+    Constructs a new QDjangoMetaModel by inspecting the given \a meta model.
 */
-QDjangoMetaModel::QDjangoMetaModel(const QObject *model)
+QDjangoMetaModel::QDjangoMetaModel(const QMetaObject *meta)
     : d(new QDjangoMetaModelPrivate)
 {
-    if (!model)
+    if (!meta)
         return;
 
-    const QMetaObject* meta = model->metaObject();
-    d->table = QString(meta->className()).toLower().toLatin1();
+    d->table = QString::fromLatin1(meta->className()).toLower();
 
     // parse table options
     const int optionsIndex = meta->indexOfClassInfo("__meta__");
@@ -162,15 +256,17 @@ QDjangoMetaModel::QDjangoMetaModel(const QObject *model)
         QMapIterator<QString, QString> option(options);
         while (option.hasNext()) {
             option.next();
-            if (option.key() == "db_table")
+            if (option.key() == QLatin1String("db_table"))
                 d->table = option.value();
+            else if (option.key() == QLatin1String("unique_together"))
+                d->uniqueTogether = option.value().toLatin1().split(',');
         }
     }
 
     const int count = meta->propertyCount();
     for(int i = QObject::staticMetaObject.propertyCount(); i < count; ++i)
     {
-        QString typeName = meta->property(i).typeName();
+        const QString typeName = QString::fromLatin1(meta->property(i).typeName());
         if (!qstrcmp(meta->property(i).name(), "pk"))
             continue;
 
@@ -183,6 +279,8 @@ QDjangoMetaModel::QDjangoMetaModel(const QObject *model)
         bool primaryKeyOption = false;
         bool nullOption = false;
         bool uniqueOption = false;
+        bool blankOption = false;
+        ForeignKeyConstraint deleteConstraint = NoAction;
         const int infoIndex = meta->indexOfClassInfo(meta->property(i).name());
         if (infoIndex >= 0)
         {
@@ -190,23 +288,34 @@ QDjangoMetaModel::QDjangoMetaModel(const QObject *model)
             QMapIterator<QString, QString> option(options);
             while (option.hasNext()) {
                 option.next();
+                const QString key = option.key();
                 const QString value = option.value();
-                if (option.key() == "auto_increment")
-                    autoIncrementOption = (value.toLower() == "true" || value == "1");
-                else if (option.key() == "db_column")
+                if (key == QLatin1String("auto_increment"))
+                    autoIncrementOption = stringToBool(value);
+                else if (key == QLatin1String("db_column"))
                     dbColumnOption = value;
-                else if (option.key() == "db_index")
-                    dbIndexOption = (value.toLower() == "true" || value == "1");
-                else if (option.key() == "ignore_field")
-                    ignoreFieldOption = (value.toLower() == "true" || value == "1");
-                else if (option.key() == "max_length")
+                else if (key == QLatin1String("db_index"))
+                    dbIndexOption = stringToBool(value);
+                else if (key == QLatin1String("ignore_field"))
+                    ignoreFieldOption = stringToBool(value);
+                else if (key == QLatin1String("max_length"))
                     maxLengthOption = value.toInt();
-                else if (option.key() == "null")
-                    nullOption = (value.toLower() == "true" || value == "1");
-                else if (option.key() == "primary_key")
-                    primaryKeyOption = (value.toLower() == "true" || value == "1");
-                else if (option.key() == "unique")
-                    uniqueOption = (value.toLower() == "true" || value == "1");
+                else if (key == QLatin1String("null"))
+                    nullOption = stringToBool(value);
+                else if (key == QLatin1String("primary_key"))
+                    primaryKeyOption = stringToBool(value);
+                else if (key == QLatin1String("unique"))
+                    uniqueOption = stringToBool(value);
+                else if (key == QLatin1String("blank"))
+                    blankOption = stringToBool(value);
+                else if (option.key() == "on_delete") {
+                    if (value.toLower() == "cascade")
+                        deleteConstraint = Cascade;
+                    else if (value.toLower() == "set_null")
+                        deleteConstraint = SetNull;
+                    else if (value.toLower() == "restrict")
+                        deleteConstraint = Restrict;
+                }
             }
         }
 
@@ -215,10 +324,9 @@ QDjangoMetaModel::QDjangoMetaModel(const QObject *model)
             continue;
 
         // foreign field
-        if (typeName.endsWith("*"))
-        {
+        if (typeName.endsWith(QLatin1Char('*'))) {
             const QByteArray fkName = meta->property(i).name();
-            const QString fkModel = typeName.left(typeName.size() - 1);
+            const QByteArray fkModel = typeName.left(typeName.size() - 1).toLatin1();
             d->foreignFields.insert(fkName, fkModel);
 
             QDjangoMetaField field;
@@ -231,6 +339,7 @@ QDjangoMetaModel::QDjangoMetaModel(const QObject *model)
             field.d->db_column = dbColumnOption.isEmpty() ? QString::fromLatin1(field.d->name) : dbColumnOption;
             field.d->index = true;
             field.d->null = nullOption;
+            field.d->deleteConstraint = deleteConstraint;
             d->localFields << field;
             continue;
         }
@@ -240,15 +349,17 @@ QDjangoMetaModel::QDjangoMetaModel(const QObject *model)
         field.d->name = meta->property(i).name();
         field.d->type = meta->property(i).type();
         field.d->db_column = dbColumnOption.isEmpty() ? QString::fromLatin1(field.d->name) : dbColumnOption;
-        field.d->index = dbIndexOption;
         field.d->maxLength = maxLengthOption;
         field.d->null = nullOption;
         if (primaryKeyOption) {
             field.d->autoIncrement = autoIncrementOption;
-            field.d->unique = true;
             d->primaryKey = field.d->name;
         } else if (uniqueOption) {
             field.d->unique = true;
+        } else if (blankOption) {
+            field.d->blank = true;
+        } else if (dbIndexOption) {
+            field.d->index = true;
         }
 
         d->localFields << field;
@@ -259,9 +370,8 @@ QDjangoMetaModel::QDjangoMetaModel(const QObject *model)
         QDjangoMetaField field;
         field.d->name = "id";
         field.d->type = QVariant::Int;
-        field.d->db_column = "id";
+        field.d->db_column = QLatin1String("id");
         field.d->autoIncrement = true;
-        field.d->unique = true;
         d->localFields.prepend(field);
         d->primaryKey = field.d->name;
     }
@@ -296,11 +406,26 @@ QDjangoMetaModel& QDjangoMetaModel::operator=(const QDjangoMetaModel& other)
     Creates the database table for this QDjangoMetaModel.
 */
 bool QDjangoMetaModel::createTable() const
+{
+    QDjangoQuery createQuery(QDjango::database());
+    foreach (const QString &sql, createTableSql()) {
+        if (!createQuery.exec(sql))
+            return false;
+    }
+    return true;
+}
+
+/*!
+    Returns the SQL queries to create the database table for this
+    QDjangoMetaModel.
+*/
+QStringList QDjangoMetaModel::createTableSql() const
 {
     QSqlDatabase db = QDjango::database();
     QSqlDriver *driver = db.driver();
     const QString driverName = db.driverName();
 
+    QStringList queries;
     QStringList propSql;
     const QString quotedTable = db.driver()->escapeIdentifier(d->table, QSqlDriver::TableName);
     foreach (const QDjangoMetaField &field, d->localFields)
@@ -308,43 +433,46 @@ bool QDjangoMetaModel::createTable() const
         QString fieldSql = driver->escapeIdentifier(field.column(), QSqlDriver::FieldName);
         switch (field.d->type) {
         case QVariant::Bool:
-            fieldSql += " BOOLEAN";
+            if (driverName == QLatin1String("QPSQL"))
+                fieldSql += QLatin1String(" boolean");
+            else
+                fieldSql += QLatin1String(" bool");
             break;
         case QVariant::ByteArray:
             if (driverName == QLatin1String("QPSQL"))
-                fieldSql += " BYTEA";
+                fieldSql += QLatin1String(" bytea");
             else {
-                fieldSql += " BLOB";
+                fieldSql += QLatin1String(" blob");
                 if (field.d->maxLength > 0)
-                    fieldSql += QString("(%1)").arg(field.d->maxLength);
+                    fieldSql += QLatin1Char('(') + QString::number(field.d->maxLength) + QLatin1Char(')');
             }
             break;
         case QVariant::Date:
-            fieldSql += " DATE";
+            fieldSql += QLatin1String(" date");
             break;
         case QVariant::DateTime:
             if (driverName == QLatin1String("QPSQL"))
-                fieldSql += " TIMESTAMP";
+                fieldSql += QLatin1String(" timestamp");
             else
-                fieldSql += " DATETIME";
+                fieldSql += QLatin1String(" datetime");
             break;
         case QVariant::Double:
-            fieldSql += " REAL";
+            fieldSql += QLatin1String(" real");
             break;
         case QVariant::Int:
-            fieldSql += " INTEGER";
+            fieldSql += QLatin1String(" integer");
             break;
         case QVariant::LongLong:
-            fieldSql += " BIGINT";
+            fieldSql += QLatin1String(" bigint");
             break;
         case QVariant::String:
             if (field.d->maxLength > 0)
-                fieldSql += QString(" VARCHAR(%1)").arg(field.d->maxLength);
+                fieldSql += QLatin1String(" varchar(") + QString::number(field.d->maxLength) + QLatin1Char(')');
             else
-                fieldSql += " TEXT";
+                fieldSql += QLatin1String(" text");
             break;
         case QVariant::Time:
-            fieldSql += " TIME";
+            fieldSql += QLatin1String(" time");
             break;
         default:
             qWarning() << "Unhandled type" << field.d->type << "for property" << field.d->name;
@@ -352,21 +480,26 @@ bool QDjangoMetaModel::createTable() const
         }
 
         if (!field.d->null)
-            fieldSql += " NOT NULL";
+            fieldSql += QLatin1String(" NOT NULL");
+        if (field.d->unique)
+            fieldSql += QLatin1String(" UNIQUE");
 
         // primary key
         if (field.d->name == d->primaryKey)
-            fieldSql += " PRIMARY KEY";
+            fieldSql += QLatin1String(" PRIMARY KEY");
 
         // auto-increment is backend specific
         if (field.d->autoIncrement) {
             if (driverName == QLatin1String("QSQLITE") ||
                 driverName == QLatin1String("QSQLITE2"))
+                // NOTE: django does not add this option for sqlite, but there
+                // is a ticket asking for it to do so:
+                // https://code.djangoproject.com/ticket/10164
                 fieldSql += QLatin1String(" AUTOINCREMENT");
             else if (driverName == QLatin1String("QMYSQL"))
                 fieldSql += QLatin1String(" AUTO_INCREMENT");
             else if (driverName == QLatin1String("QPSQL"))
-                fieldSql = driver->escapeIdentifier(field.column(), QSqlDriver::FieldName) + " SERIAL PRIMARY KEY";
+                fieldSql = driver->escapeIdentifier(field.column(), QSqlDriver::FieldName) + QLatin1String(" serial PRIMARY KEY");
         }
 
         // foreign key
@@ -374,34 +507,61 @@ bool QDjangoMetaModel::createTable() const
         {
             const QDjangoMetaModel foreignMeta = QDjango::metaModel(field.d->foreignModel);
             const QDjangoMetaField foreignField = foreignMeta.localField("pk");
-            fieldSql += QString(" REFERENCES %1 (%2)").arg(
+            fieldSql += QString::fromLatin1(" REFERENCES %1 (%2)").arg(
                 driver->escapeIdentifier(foreignMeta.d->table, QSqlDriver::TableName),
                 driver->escapeIdentifier(foreignField.column(), QSqlDriver::FieldName));
+
+            if (field.d->deleteConstraint != NoAction) {
+                fieldSql += " ON DELETE";
+                switch (field.d->deleteConstraint) {
+                case Cascade:
+                    fieldSql += " CASCADE";
+                    break;
+                case SetNull:
+                    fieldSql += " SET NULL";
+                    break;
+                case Restrict:
+                    fieldSql += " RESTRICT";
+                    break;
+                default:
+                    break;
+                }
+            }
+
+            if (driverName == QLatin1String("QPSQL"))
+                fieldSql += " DEFERRABLE INITIALLY DEFERRED";
         }
         propSql << fieldSql;
     }
 
+    // unique contraints
+    if (!d->uniqueTogether.isEmpty()) {
+        QStringList columns;
+        foreach (const QByteArray &name, d->uniqueTogether) {
+            columns << driver->escapeIdentifier(localField(name).column(), QSqlDriver::FieldName);
+        }
+        propSql << QString::fromLatin1("UNIQUE (%2)").arg(columns.join(QLatin1String(", ")));
+    }
+
     // create table
-    QDjangoQuery createQuery(db);
-    if (!createQuery.exec(QString("CREATE TABLE %1 (%2)").arg(
+    queries << QString::fromLatin1("CREATE TABLE %1 (%2)").arg(
             quotedTable,
-            propSql.join(", "))))
-        return false;
+            propSql.join(QLatin1String(", ")));
 
     // create indices
     foreach (const QDjangoMetaField &field, d->localFields) {
-        if (field.d->index && !field.d->unique) {
-            const QString indexName = d->table + "_" + field.column();
-            if (!createQuery.exec(QString("CREATE INDEX %1 ON %2 (%3)").arg(
+        if (field.d->index) {
+            const QString indexName = d->table + QLatin1Char('_')
+                + stringlist_digest(QStringList() << field.column());
+            queries << QString::fromLatin1("CREATE INDEX %1 ON %2 (%3)").arg(
                 // FIXME : how should we escape an index name?
                 driver->escapeIdentifier(indexName, QSqlDriver::FieldName),
                 quotedTable,
-                driver->escapeIdentifier(field.column(), QSqlDriver::FieldName))))
-                return false;
+                driver->escapeIdentifier(field.column(), QSqlDriver::FieldName));
         }
     }
 
-    return true;
+    return queries;
 }
 
 /*!
@@ -412,8 +572,8 @@ bool QDjangoMetaModel::dropTable() const
     QSqlDatabase db = QDjango::database();
 
     QDjangoQuery query(db);
-    return query.exec(QString("DROP TABLE %1").arg(
-        db.driver()->escapeIdentifier(d->table, QSqlDriver::TableName)));
+    return query.exec(QLatin1String("DROP TABLE ") +
+        db.driver()->escapeIdentifier(d->table, QSqlDriver::TableName));
 }
 
 /*!
@@ -424,19 +584,25 @@ bool QDjangoMetaModel::dropTable() const
 */
 QObject *QDjangoMetaModel::foreignKey(const QObject *model, const char *name) const
 {
+    // check the name is valid
     const QByteArray prop(name);
+    if (!d->foreignFields.contains(prop)) {
+        qWarning("QDjangoMetaModel cannot get foreign model for invalid key '%s'", name);
+        return 0;
+    }
+
     QObject *foreign = model->property(prop + "_ptr").value<QObject*>();
     if (!foreign)
         return 0;
 
     // if the foreign object was not loaded yet, do it now
-    const QString foreignClass = d->foreignFields[prop];
+    const QByteArray foreignClass = d->foreignFields[prop];
     const QDjangoMetaModel foreignMeta = QDjango::metaModel(foreignClass);
     const QVariant foreignPk = model->property(prop + "_id");
     if (foreign->property(foreignMeta.primaryKey()) != foreignPk)
     {
         QDjangoQuerySetPrivate qs(foreignClass);
-        qs.addFilter(QDjangoWhere("pk", QDjangoWhere::Equals, foreignPk));
+        qs.addFilter(QDjangoWhere(QLatin1String("pk"), QDjangoWhere::Equals, foreignPk));
         qs.sqlFetch();
         if (qs.properties.size() != 1 || !qs.sqlLoad(foreign, 0))
             return 0;
@@ -450,25 +616,25 @@ QObject *QDjangoMetaModel::foreignKey(const QObject *model, const char *name) co
     \param model
     \param name
     \param value
-
-    \note The \c model will take ownership of the given \c value.
 */
 void QDjangoMetaModel::setForeignKey(QObject *model, const char *name, QObject *value) const
 {
+    // check the name is valid
     const QByteArray prop(name);
+    if (!d->foreignFields.contains(prop)) {
+        qWarning("QDjangoMetaModel cannot set foreign model for invalid key '%s'", name);
+        return;
+    }
+
     QObject *old = model->property(prop + "_ptr").value<QObject*>();
     if (old == value)
         return;
-    if (old)
-        delete old;
 
     // store the new pointer and update the foreign key
     model->setProperty(prop + "_ptr", qVariantFromValue(value));
-    if (value)
-    {
+    if (value) {
         const QDjangoMetaModel foreignMeta = QDjango::metaModel(d->foreignFields[prop]);
         model->setProperty(prop + "_id", value->property(foreignMeta.primaryKey()));
-        value->setParent(model);
     } else {
         model->setProperty(prop + "_id", QVariant());
     }
@@ -500,7 +666,7 @@ void QDjangoMetaModel::load(QObject *model, const QVariantList &properties, int
 /*!
     Returns the foreign field mapping.
 */
-QMap<QByteArray, QString> QDjangoMetaModel::foreignFields() const
+QMap<QByteArray, QByteArray> QDjangoMetaModel::foreignFields() const
 {
     return d->foreignFields;
 }
@@ -508,9 +674,9 @@ QMap<QByteArray, QString> QDjangoMetaModel::foreignFields() const
 /*!
     Return the local field with the specified \a name.
 */
-QDjangoMetaField QDjangoMetaModel::localField(const QString &name) const
+QDjangoMetaField QDjangoMetaModel::localField(const char *name) const
 {
-    const QString fieldName = (name == "pk") ? d->primaryKey : name;
+    const QByteArray fieldName = strcmp(name, "pk") ? QByteArray(name) : d->primaryKey;
     foreach (const QDjangoMetaField &field, d->localFields) {
         if (field.d->name == fieldName)
             return field;
@@ -549,7 +715,7 @@ bool QDjangoMetaModel::remove(QObject *model) const
 {
     const QVariant pk = model->property(d->primaryKey);
     QDjangoQuerySetPrivate qs(model->metaObject()->className());
-    qs.addFilter(QDjangoWhere("pk", QDjangoWhere::Equals, pk));
+    qs.addFilter(QDjangoWhere(QLatin1String("pk"), QDjangoWhere::Equals, pk));
     return qs.sqlDelete();
 }
 
@@ -567,7 +733,7 @@ bool QDjangoMetaModel::save(QObject *model) const
     {
         QSqlDatabase db = QDjango::database();
         QDjangoQuery query(db);
-        query.prepare(QString("SELECT 1 AS a FROM %1 WHERE %2 = ?").arg(
+        query.prepare(QString::fromLatin1("SELECT 1 AS a FROM %1 WHERE %2 = ?").arg(
                       db.driver()->escapeIdentifier(d->table, QSqlDriver::FieldName),
                       db.driver()->escapeIdentifier(primaryKey.column(), QSqlDriver::FieldName)));
         query.addBindValue(pk);
@@ -578,13 +744,13 @@ bool QDjangoMetaModel::save(QObject *model) const
             foreach (const QDjangoMetaField &field, d->localFields) {
                 if (field.d->name != d->primaryKey) {
                     const QVariant value = model->property(field.d->name);
-                    fields.insert(field.d->name, field.toDatabase(value));
+                    fields.insert(QString::fromLatin1(field.d->name), field.toDatabase(value));
                 }
             }
 
             // perform UPDATE
             QDjangoQuerySetPrivate qs(model->metaObject()->className());
-            qs.addFilter(QDjangoWhere("pk", QDjangoWhere::Equals, pk));
+            qs.addFilter(QDjangoWhere(QLatin1String("pk"), QDjangoWhere::Equals, pk));
             return qs.sqlUpdate(fields) != -1;
         }
     }
@@ -599,14 +765,17 @@ bool QDjangoMetaModel::save(QObject *model) const
     }
 
     // perform INSERT
-    QVariant insertId;
     QDjangoQuerySetPrivate qs(model->metaObject()->className());
-    if (!qs.sqlInsert(fields, &insertId))
-        return false;
-
-    // fetch autoincrement pk
-    if (primaryKey.d->autoIncrement)
+    if (primaryKey.d->autoIncrement) {
+        // fetch autoincrement pk
+        QVariant insertId;
+        if (!qs.sqlInsert(fields, &insertId))
+            return false;
         model->setProperty(d->primaryKey, insertId);
+    } else {
+        if (!qs.sqlInsert(fields))
+            return false;
+    }
     return true;
 }
 
